@@ -520,7 +520,7 @@ class CoincsDocument(object):
 
 
 class Data(object):
-	def __init__(self, filename, process_params, pipeline, instruments, seg, coincidence_threshold, coinc_params_distributions, ranking_data, marginalized_likelihood_file = None, likelihood_files_namedtuple = None, injection_filename = None, time_slide_file = None, comment = None, tmp_path = None, likelihood_snapshot_interval = None, thinca_interval = 50.0, sngls_snr_threshold = None, gracedb_far_threshold = None, gracedb_group = "Test", gracedb_search = "LowMass", gracedb_pipeline = "gstlal", gracedb_service_url = "https://gracedb.ligo.org/api/", replace_file = True, verbose = False):
+	def __init__(self, filename, process_params, pipeline, instruments, seg, coincidence_threshold, coinc_params_distributions, zero_lag_ranking_stats = None, marginalized_likelihood_file = None, likelihood_files_namedtuple = None, injection_filename = None, time_slide_file = None, comment = None, tmp_path = None, likelihood_snapshot_interval = None, thinca_interval = 50.0, sngls_snr_threshold = None, gracedb_far_threshold = None, gracedb_group = "Test", gracedb_search = "LowMass", gracedb_pipeline = "gstlal", gracedb_service_url = "https://gracedb.ligo.org/api/", replace_file = True, upload_auxiliary_data_to_gracedb = True, verbose = False):
 		#
 		# initialize
 		#
@@ -528,6 +528,7 @@ class Data(object):
 		self.lock = threading.Lock()
 		self.pipeline = pipeline
 		self.verbose = verbose
+		self.upload_auxiliary_data_to_gracedb = upload_auxiliary_data_to_gracedb
 		# None to disable likelihood ratio assignment, otherwise a filename
 		self.marginalized_likelihood_file = marginalized_likelihood_file
 		self.likelihood_files_namedtuple = likelihood_files_namedtuple
@@ -551,6 +552,7 @@ class Data(object):
 		bottle.route("/snr_history.txt")(self.web_get_snr_history)
 		bottle.route("/ram_history.txt")(self.web_get_ram_history)
 		bottle.route("/likelihood.xml")(self.web_get_likelihood_file)
+		bottle.route("/zero_lag_ranking_stats.xml")(self.web_get_zero_lag_ranking_stats_file)
 		bottle.route("/gracedb_far_threshold.txt", method = "GET")(self.web_get_gracedb_far_threshold)
 		bottle.route("/gracedb_far_threshold.txt", method = "POST")(self.web_set_gracedb_far_threshold)
 		bottle.route("/sngls_snr_threshold.txt", method = "GET")(self.web_get_sngls_snr_threshold)
@@ -574,12 +576,24 @@ class Data(object):
 
 		#
 		# setup likelihood ratio book-keeping.  seglistsdicts to be
-		# populated the pipeline handler
+		# populated the pipeline handler.  zero_lag_ranking_stats
+		# is a RankingData object that is used to accumulate a
+		# histogram of the likelihood ratio values assigned to
+		# zero-lag candidates.  this is required to implement the
+		# extinction model for low-significance events during
+		# online running but otherwise is optional.  ranking_data
+		# contains the RankingData object used to initialize the
+		# FAPFAR object for on-the-fly FAP and FAR assignment.
+		# except to initialize the FAPFAR object it is not used for
+		# anything, but is retained so that it can be exposed
+		# through the web interface for diagnostic purposed and
+		# uploaded to gracedb with candidates.
 		#
 
 		self.coinc_params_distributions = coinc_params_distributions
-		self.ranking_data = ranking_data
+		self.ranking_data = None
 		self.seglistdicts = None
+		self.zero_lag_ranking_stats = zero_lag_ranking_stats
 		self.fapfar = None
 
 		#
@@ -620,23 +634,9 @@ class Data(object):
 			if self.likelihood_snapshot_interval is not None and (self.likelihood_snapshot_timestamp is None or buf_timestamp - self.likelihood_snapshot_timestamp >= self.likelihood_snapshot_interval):
 				self.likelihood_snapshot_timestamp = buf_timestamp
 
-				# smooth the distributions.  re-populates
-				# PDF arrays from raw counts
-				self.coinc_params_distributions.finish(verbose = self.verbose)
-
-				# post a checkpoint message.  FIXME:  make
-				# sure this triggers
-				# self.snapshot_output_file() to be
-				# invoked.  lloidparts takes care of that
-				# for now, but spreading the program logic
-				# around like that isn't a good idea, this
-				# code should be responsible for it
-				# somehow, no?
-				self.pipeline.get_bus().post(message_new_checkpoint(self.pipeline, timestamp = buf_timestamp.ns()))
-
-				# If a reference likelihood file is given, load
-				# data coinc_params_distributions data before
-				# smoothing distributions
+				# if a reference likelihood file is given,
+				# overwrite coinc_params_distributions with
+				# its contents
 				# FIXME There is currently no guarantee that
 				# the reference_likelihood_file on disk will
 				# have updated since the last snapshot, but for
@@ -645,7 +645,28 @@ class Data(object):
 				# older than the snapshot before last
 				if self.likelihood_files_namedtuple.reference_likelihood_file is not None:
 					self.coinc_params_distributions = far.parse_likelihood_control_doc(ligolw_utils.load_filename(self.likelihood_files_namedtuple.reference_likelihood_file, verbose = self.verbose, contenthandler = far.ThincaCoincParamsDistributions.LIGOLWContentHandler))[0]
-					self.coinc_params_distributions.finish(verbose = self.verbose)
+
+				# smooth the distributions.  re-populates
+				# PDF arrays from raw counts
+				self.coinc_params_distributions.finish(verbose = self.verbose)
+
+				# post a checkpoint message.
+				# FIXME:  make sure this triggers
+				# self.snapshot_output_file() to be
+				# invoked.  lloidparts takes care of that
+				# for now, but spreading the program logic
+				# around like that isn't a good idea, this
+				# code should be responsible for it
+				# somehow, no?  NOTE:
+				# self.snapshot_output_file() does not
+				# write the coinc_params_distributions
+				# object to disk if a reference likelihood
+				# file is given, so the the thing that was
+				# just read in is not written back out
+				# again.  see the comment in that function
+				# about turning it into two handlers and
+				# only hooking up which ones are needed.
+				self.pipeline.get_bus().post(message_new_checkpoint(self.pipeline, timestamp = buf_timestamp.ns()))
 
 				if self.marginalized_likelihood_file is not None:
 					# FIXME:  must set horizon
@@ -662,11 +683,11 @@ class Data(object):
 					# been updated asynchronously and
 					# initialize a FAP/FAR assignment
 					# machine from it.
-					ranking_data, seglists = far.parse_likelihood_control_doc(ligolw_utils.load_filename(self.marginalized_likelihood_file, verbose = self.verbose, contenthandler = far.ThincaCoincParamsDistributions.LIGOLWContentHandler))[1:]
-					if ranking_data is None:
+					self.ranking_data, seglists = far.parse_likelihood_control_doc(ligolw_utils.load_filename(self.marginalized_likelihood_file, verbose = self.verbose, contenthandler = far.ThincaCoincParamsDistributions.LIGOLWContentHandler))[1:]
+					if self.ranking_data is None:
 						raise ValueError("\"%s\" does not contain ranking statistic PDFs" % self.marginalized_likelihood_file)
-					ranking_data.finish(verbose = self.verbose)
-					self.fapfar = far.FAPFAR(ranking_data, livetime = far.get_live_time(seglists))
+					self.ranking_data.finish(verbose = self.verbose)
+					self.fapfar = far.FAPFAR(self.ranking_data, livetime = far.get_live_time(seglists))
 
 			# run stream thinca.  update the parameter
 			# distribution data from sngls that weren't used in
@@ -699,15 +720,16 @@ class Data(object):
 			if self.stream_thinca.last_coincs:
 				self.stream_thinca.last_coincs.coinc_event_index = dict([max(self.stream_thinca.last_coincs.coinc_event_index.iteritems(), key = lambda (coinc_event_id, coinc_event): coinc_event.likelihood)])
 
-			# Add events to the observed likelihood histogram post "clustering"
-			# FIXME proper clustering is really needed (see above)
-			if self.stream_thinca.last_coincs:
+			# Add events to the observed likelihood histogram
+			# post "clustering"
+			# FIXME proper clustering is really needed (see
+			# above)
+			if self.stream_thinca.last_coincs and self.zero_lag_ranking_stats is not None:
 				for coinc_event_id, coinc_event in self.stream_thinca.last_coincs.coinc_event_index.items():
 					offset_vector = self.stream_thinca.last_coincs.offset_vector(coinc_event.time_slide_id)
-					#IFOS come from coinc_inspiral not coinc_event
-					ifos = self.stream_thinca.last_coincs.coinc_inspiral_index[coinc_event_id].ifos
+					instruments = frozenset(self.stream_thinca.last_coincs.coinc_inspiral_index[coinc_event_id].instruments)
 					if (coinc_event.likelihood is not None and coinc_event.likelihood >= far.RankingData.ln_likelihood_ratio_threshold) and not any(offset_vector.values()):
-						self.ranking_data.zero_lag_likelihood_rates[frozenset(lsctables.instrument_set_from_ifos(ifos))][coinc_event.likelihood,] += 1
+						self.zero_lag_ranking_stats.zero_lag_likelihood_rates[instruments][coinc_event.likelihood,] += 1
 
 			# do GraceDB alerts
 			if self.gracedb_far_threshold is not None:
@@ -752,6 +774,25 @@ class Data(object):
 			output.close()
 			return outstr
 
+	def __get_zero_lag_ranking_stats_file(self):
+		xmldoc = ligolw.Document()
+		xmldoc.appendChild(ligolw.LIGO_LW())
+		process = ligolw_process.register_to_xmldoc(xmldoc, u"gstlal_inspiral", paramdict = {})
+		search_summary = ligolw_search_summary.append_search_summary(xmldoc, process, ifos = self.seglistdicts["triggersegments"].keys(), inseg = self.seglistdicts["triggersegments"].extent_all(), outseg = self.seglistdicts["triggersegments"].extent_all())
+		# FIXME:  now that we've got all kinds of segment lists
+		# being collected, decide which of them should go here.
+		far.gen_likelihood_control_doc(xmldoc, process, None, self.zero_lag_ranking_stats, self.seglistdicts["triggersegments"])
+		ligolw_process.set_process_end_time(process)
+		return xmldoc
+
+	def web_get_zero_lag_ranking_stats_file(self):
+		with self.lock:
+			output = StringIO.StringIO()
+			ligolw_utils.write_fileobj(self.__get_zero_lag_ranking_stats_file(), output, trap_signals = None)
+			outstr = output.getvalue()
+			output.close()
+			return outstr
+
 	def __flush(self):
 		# run StreamThinca's .flush().  returns the last remaining
 		# non-coincident sngls.  add them to the distribution
@@ -776,15 +817,16 @@ class Data(object):
 		if self.stream_thinca.last_coincs:
 			self.stream_thinca.last_coincs.coinc_event_index = dict([max(self.stream_thinca.last_coincs.coinc_event_index.iteritems(), key = lambda (coinc_event_id, coinc_event): coinc_event.likelihood)])
 
-		# Add events to the observed likelihood histogram post "clustering"
+		# Add events to the observed likelihood histogram post
+		# "clustering"
 		# FIXME proper clustering is really needed (see above)
-		if self.stream_thinca.last_coincs:
+		if self.stream_thinca.last_coincs and self.zero_lag_ranking_stats is not None:
 			for coinc_event_id, coinc_event in self.stream_thinca.last_coincs.coinc_event_index.items():
 				offset_vector = self.stream_thinca.last_coincs.offset_vector(coinc_event.time_slide_id)
-				#IFOS come from coinc_inspiral not coinc_event
-				ifos = self.stream_thinca.last_coincs.coinc_inspiral_index[coinc_event_id].ifos
+				instruments = frozenset(self.stream_thinca.last_coincs.coinc_inspiral_index[coinc_event_id].instruments)
 				if (coinc_event.likelihood is not None and coinc_event.likelihood >= far.RankingData.ln_likelihood_ratio_threshold) and not any(offset_vector.values()):
-					self.ranking_data.zero_lag_likelihood_rates[frozenset(lsctables.instrument_set_from_ifos(ifos))][coinc_event.likelihood,] += 1
+					self.zero_lag_ranking_stats.zero_lag_likelihood_rates[instruments][coinc_event.likelihood,] += 1
+
 
 		# do GraceDB alerts
 		if self.gracedb_far_threshold is not None:
@@ -794,125 +836,160 @@ class Data(object):
 		with self.lock:
 			self.__flush()
 
-	def __do_gracedb_alerts(self):
-		if self.stream_thinca.last_coincs:
-			gracedb_client = gracedb.rest.GraceDb(self.gracedb_service_url)
-			gracedb_ids = []
-			common_messages = []
-			coinc_inspiral_index = self.stream_thinca.last_coincs.coinc_inspiral_index
+	def __do_gracedb_alerts(self, retries = 5, retry_delay = 5.):
+		# no-op short circuit
+		if not self.stream_thinca.last_coincs:
+			return
 
-			# This appears to be a silly for loop since
-			# coinc_event_index will only have one value, but we're
-			# future proofing this at the point where it could have
-			# multiple clustered events
-			for coinc_event in self.stream_thinca.last_coincs.coinc_event_index.values():
-				#
-				# continue if the false alarm rate is not low
-				# enough, or is nan.
-				#
+		gracedb_client = gracedb.rest.GraceDb(self.gracedb_service_url)
+		gracedb_ids = []
+		common_messages = []
+		coinc_inspiral_index = self.stream_thinca.last_coincs.coinc_inspiral_index
 
-				if coinc_inspiral_index[coinc_event.coinc_event_id].combined_far is None or coinc_inspiral_index[coinc_event.coinc_event_id].combined_far > self.gracedb_far_threshold or numpy.isnan(coinc_inspiral_index[coinc_event.coinc_event_id].combined_far):
-					continue
+		# This appears to be a silly for loop since
+		# coinc_event_index will only have one value, but we're
+		# future proofing this at the point where it could have
+		# multiple clustered events
+		for coinc_event in self.stream_thinca.last_coincs.coinc_event_index.values():
+			#
+			# continue if the false alarm rate is not low
+			# enough, or is nan.
+			#
 
-				#
-				# retrieve PSDs and ranking data
-				#
+			if coinc_inspiral_index[coinc_event.coinc_event_id].combined_far is None or coinc_inspiral_index[coinc_event.coinc_event_id].combined_far > self.gracedb_far_threshold or numpy.isnan(coinc_inspiral_index[coinc_event.coinc_event_id].combined_far):
+				continue
 
-				if not common_messages:
-					if self.verbose:
-						print >>sys.stderr, "retrieving PSDs from whiteners and generating psd.xml.gz ..."
-					psddict = {}
-					for instrument in self.seglistdicts["triggersegments"]:
-						elem = self.pipeline.get_by_name("lal_whiten_%s" % instrument)
-						# FIXME:  remove
-						# LIGOTimeGPS type cast
-						# when we port to swig
-						# version of
-						# REAL8FrequencySeries
-						psddict[instrument] = REAL8FrequencySeries(
-							name = "PSD",
-							epoch = LIGOTimeGPS(lal.UTCToGPS(time.gmtime()), 0),
-							f0 = 0.0,
-							deltaF = elem.get_property("delta-f"),
-							sampleUnits = LALUnit("s strain^2"),	# FIXME:  don't hard-code this
-							data = numpy.array(elem.get_property("mean-psd"))
-						)
-					fobj = StringIO.StringIO()
-					reference_psd.write_psd_fileobj(fobj, psddict, gz = True, trap_signals = None)
-					common_messages.append(("strain spectral densities", "psd.xml.gz", "psd", fobj.getvalue()))
+			#
+			# retrieve PSDs and ranking data
+			#
 
-					if self.verbose:
-						print >>sys.stderr, "generating ranking_data.xml.gz ..."
-					fobj = StringIO.StringIO()
-					ligolw_utils.write_fileobj(self.__get_likelihood_file(), fobj, gz = True, trap_signals = None)
-					common_messages.append(("ranking statistic PDFs", "ranking_data.xml.gz", "ranking statistic", fobj.getvalue()))
-					del fobj
-
-				#
-				# fake a filename for end-user convenience
-				#
-
-				observatories = "".join(sorted(set(instrument[0] for instrument in self.seglistdicts["triggersegments"])))
-				instruments = "".join(sorted(self.seglistdicts["triggersegments"]))
-				description = "%s_%s_%s_%s" % (instruments, ("%.4g" % coinc_inspiral_index[coinc_event.coinc_event_id].mass).replace(".", "_").replace("-", "_"), self.gracedb_group, self.gracedb_search)
-				end_time = int(coinc_inspiral_index[coinc_event.coinc_event_id].end)
-				filename = "%s-%s-%d-%d.xml" % (observatories, description, end_time, 0)
-
-				#
-				# construct message and send to gracedb.
-				# we go through the intermediate step of
-				# first writing the document into a string
-				# buffer incase there is some safety in
-				# doing so in the event of a malformed
-				# document;  instead of writing directly
-				# into gracedb's input pipe and crashing
-				# part way through.
-				#
+			if not common_messages and self.upload_auxiliary_data_to_gracedb:
+				if self.verbose:
+					print >>sys.stderr, "retrieving PSDs from whiteners and generating psd.xml.gz ..."
+				psddict = {}
+				for instrument in self.seglistdicts["triggersegments"]:
+					elem = self.pipeline.get_by_name("lal_whiten_%s" % instrument)
+					# FIXME:  remove
+					# LIGOTimeGPS type cast
+					# when we port to swig
+					# version of
+					# REAL8FrequencySeries
+					psddict[instrument] = REAL8FrequencySeries(
+						name = "PSD",
+						epoch = LIGOTimeGPS(lal.UTCToGPS(time.gmtime()), 0),
+						f0 = 0.0,
+						deltaF = elem.get_property("delta-f"),
+						sampleUnits = LALUnit("s strain^2"),	# FIXME:  don't hard-code this
+						data = numpy.array(elem.get_property("mean-psd"))
+					)
+				fobj = StringIO.StringIO()
+				reference_psd.write_psd_fileobj(fobj, psddict, gz = True, trap_signals = None)
+				common_messages.append(("strain spectral densities", "psd.xml.gz", "psd", fobj.getvalue()))
 
 				if self.verbose:
-					print >>sys.stderr, "sending %s to gracedb ..." % filename
-				message = StringIO.StringIO()
-				xmldoc = self.stream_thinca.last_coincs[coinc_event.coinc_event_id]
-				# give the alert all the standard inspiral
-				# columns (attributes should all be
-				# populated).  FIXME:  ugly.
-				sngl_inspiral_table = lsctables.SnglInspiralTable.get_table(xmldoc)
-				for standard_column in ("process_id", "ifo", "search", "channel", "end_time", "end_time_ns", "end_time_gmst", "impulse_time", "impulse_time_ns", "template_duration", "event_duration", "amplitude", "eff_distance", "coa_phase", "mass1", "mass2", "mchirp", "mtotal", "eta", "kappa", "chi", "tau0", "tau2", "tau3", "tau4", "tau5", "ttotal", "psi0", "psi3", "alpha", "alpha1", "alpha2", "alpha3", "alpha4", "alpha5", "alpha6", "beta", "f_final", "snr", "chisq", "chisq_dof", "bank_chisq", "bank_chisq_dof", "cont_chisq", "cont_chisq_dof", "sigmasq", "rsqveto_duration", "Gamma0", "Gamma1", "Gamma2", "Gamma3", "Gamma4", "Gamma5", "Gamma6", "Gamma7", "Gamma8", "Gamma9", "spin1x", "spin1y", "spin1z", "spin2x", "spin2y", "spin2z", "event_id"):
+					print >>sys.stderr, "generating ranking_data.xml.gz ..."
+				fobj = StringIO.StringIO()
+				ligolw_utils.write_fileobj(self.__get_likelihood_file(), fobj, gz = True, trap_signals = None)
+				common_messages.append(("ranking statistic PDFs", "ranking_data.xml.gz", "ranking statistic", fobj.getvalue()))
+
+				# FIXME:  stop doing this when
+				# .new_with_extinction() patch is applied.
+				# right now the ranking statistic PDF and
+				# CCDF uploaded to gracedb is missing the
+				# extinction model and and the zero-lag
+				# histogram is required to construct it.
+				# after .new_with_extinction() what gest
+				# uploaded will have had the extinction
+				# model applied and this auxiliary
+				# information will no longer be required
+				if self.verbose:
+					print >>sys.stderr, "generating zero_lag_ranking_stats.xml.gz ..."
+				fobj = StringIO.StringIO()
+				ligolw_utils.write_fileobj(self.__get_zero_lag_ranking_stats_file(), fobj, gz = True, trap_signals = None)
+				common_messages.append(("zero-lag ranking statistic histogram", "zero_lag_ranking_stats.xml.gz", "ranking statistic", fobj.getvalue()))
+				del fobj
+
+			#
+			# fake a filename for end-user convenience
+			#
+
+			observatories = "".join(sorted(set(instrument[0] for instrument in self.seglistdicts["triggersegments"])))
+			instruments = "".join(sorted(self.seglistdicts["triggersegments"]))
+			description = "%s_%s_%s_%s" % (instruments, ("%.4g" % coinc_inspiral_index[coinc_event.coinc_event_id].mass).replace(".", "_").replace("-", "_"), self.gracedb_group, self.gracedb_search)
+			end_time = int(coinc_inspiral_index[coinc_event.coinc_event_id].end)
+			filename = "%s-%s-%d-%d.xml" % (observatories, description, end_time, 0)
+
+			#
+			# construct message and send to gracedb.
+			# we go through the intermediate step of
+			# first writing the document into a string
+			# buffer incase there is some safety in
+			# doing so in the event of a malformed
+			# document;  instead of writing directly
+			# into gracedb's input pipe and crashing
+			# part way through.
+			#
+
+			if self.verbose:
+				print >>sys.stderr, "sending %s to gracedb ..." % filename
+			message = StringIO.StringIO()
+			xmldoc = self.stream_thinca.last_coincs[coinc_event.coinc_event_id]
+			# give the alert all the standard inspiral
+			# columns (attributes should all be
+			# populated).  FIXME:  ugly.
+			sngl_inspiral_table = lsctables.SnglInspiralTable.get_table(xmldoc)
+			for standard_column in ("process_id", "ifo", "search", "channel", "end_time", "end_time_ns", "end_time_gmst", "impulse_time", "impulse_time_ns", "template_duration", "event_duration", "amplitude", "eff_distance", "coa_phase", "mass1", "mass2", "mchirp", "mtotal", "eta", "kappa", "chi", "tau0", "tau2", "tau3", "tau4", "tau5", "ttotal", "psi0", "psi3", "alpha", "alpha1", "alpha2", "alpha3", "alpha4", "alpha5", "alpha6", "beta", "f_final", "snr", "chisq", "chisq_dof", "bank_chisq", "bank_chisq_dof", "cont_chisq", "cont_chisq_dof", "sigmasq", "rsqveto_duration", "Gamma0", "Gamma1", "Gamma2", "Gamma3", "Gamma4", "Gamma5", "Gamma6", "Gamma7", "Gamma8", "Gamma9", "spin1x", "spin1y", "spin1z", "spin2x", "spin2y", "spin2z", "event_id"):
+				try:
+					sngl_inspiral_table.appendColumn(standard_column)
+				except ValueError:
+					# already has it
+					pass
+			ligolw_utils.write_fileobj(xmldoc, message, gz = False, trap_signals = None)
+			xmldoc.unlink()
+			# FIXME: make this optional from command line?
+			if True:
+				for attempt in range(1, retries + 1):
 					try:
-						sngl_inspiral_table.appendColumn(standard_column)
-					except ValueError:
-						# already has it
+						resp = gracedb_client.createEvent(self.gracedb_group, self.gracedb_pipeline, filename, filecontents = message.getvalue(), search = self.gracedb_search)
+					except gracedb.rest.HTTPError as resp:
 						pass
-				ligolw_utils.write_fileobj(xmldoc, message, gz = False, trap_signals = None)
-				xmldoc.unlink()
-				# FIXME: make this optional from command line?
-				if True:
-					resp = gracedb_client.createEvent(self.gracedb_group, self.gracedb_pipeline, filename, filecontents = message.getvalue(), search = self.gracedb_search)
-					resp_json = resp.json()
-					if resp.status != httplib.CREATED:
-						print >>sys.stderr, "gracedb upload of %s failed" % filename
 					else:
-						if self.verbose:
-							print >>sys.stderr, "event assigned grace ID %s" % resp_json["graceid"]
-						gracedb_ids.append(resp_json["graceid"])
+						resp_json = resp.json()
+						if resp.status == httplib.CREATED:
+							if self.verbose:
+								print >>sys.stderr, "event assigned grace ID %s" % resp_json["graceid"]
+							gracedb_ids.append(resp_json["graceid"])
+							break
+					print >>sys.stderr, "gracedb upload of %s failed on attempt %d/%d: %d: %s"  % (filename, attempt, retries, resp.status, httplib.responses.get(resp.status, "Unknown"))
+					time.sleep(random.lognormal(math.log(retry_delay), .5))
 				else:
-					proc = subprocess.Popen(("/bin/cp", "/dev/stdin", filename), stdin = subprocess.PIPE)
-					proc.stdin.write(message.getvalue())
-					proc.stdin.flush()
-					proc.stdin.close()
-				message.close()
+					print >>sys.stderr, "gracedb upload of %s failed" % filename
+			else:
+				proc = subprocess.Popen(("/bin/cp", "/dev/stdin", filename), stdin = subprocess.PIPE)
+				proc.stdin.write(message.getvalue())
+				proc.stdin.flush()
+				proc.stdin.close()
+			message.close()
 
-			#
-			# do PSD and ranking data file uploads
-			#
+		#
+		# do PSD and ranking data file uploads
+		#
 
-			while common_messages:
-				message, filename, tag, contents = common_messages.pop()
-				for gracedb_id in gracedb_ids:
-					resp = gracedb_client.writeLog(gracedb_id, message, filename = filename, filecontents = contents, tagname = tag)
-					resp_json = resp.json()
-					if resp.status != httplib.CREATED:
-						print >>sys.stderr, "gracedb upload of %s for ID %s failed" % (filename, gracedb_id)
+		while common_messages:
+			message, filename, tag, contents = common_messages.pop()
+			for gracedb_id in gracedb_ids:
+				for attempt in range(1, retries + 1):
+					try:
+						resp = gracedb_client.writeLog(gracedb_id, message, filename = filename, filecontents = contents, tagname = tag)
+					except gracedb.rest.HTTPError as resp:
+						pass
+					else:
+						if resp.status == httplib.CREATED:
+							break
+					print >>sys.stderr, "gracedb upload of %s for ID %s failed on attempt %d/%d: %d: %s"  % (filename, gracedb_id, attempt, retries, resp.status, httplib.responses.get(resp.status, "Unknown"))
+					time.sleep(random.lognormal(math.log(retry_delay), .5))
+				else:
+					print >>sys.stderr, "gracedb upload of %s for ID %s failed" % (filename, gracedb_id)
 
 	def do_gracedb_alerts(self):
 		with self.lock:
@@ -1039,6 +1116,9 @@ class Data(object):
 			fname = self.coincs_document.T050017_filename(description + '_DISTSTATS', 'xml.gz')
 			shutil.copy(os.path.join(path,filename), os.path.join(subdir_from_T050017_filename(fname), fname))
 
+	def __write_zero_lag_ranking_stats_file(self, filename, verbose = False):
+		ligolw_utils.write_filename(self.__get_zero_lag_ranking_stats_file(), filename, gz = (filename or "stdout").endswith(".gz"), verbose = verbose, trap_signals = None)
+
 	def write_output_file(self, filename = None, description = "", verbose = False):
 		with self.lock:
 			self.__write_output_file(filename = filename, verbose = verbose)
@@ -1048,7 +1128,7 @@ class Data(object):
 			# can't be used anymore
 			del self.coincs_document
 
-	def snapshot_output_file(self, description, extension, verbose = False):
+	def snapshot_output_file(self, description, extension, zero_lag_ranking_stats_filename = None, verbose = False):
 		with self.lock:
 			coincs_document = self.coincs_document.get_another()
 			# We require the likelihood file to have the same name
@@ -1059,6 +1139,8 @@ class Data(object):
 			self.__write_output_file(filename = fname, verbose = verbose)
 			if self.likelihood_files_namedtuple.likelihood_file:
 				self.__write_likelihood_file(self.likelihood_files_namedtuple.likelihood_file, description, snapshot = True, verbose = verbose)
+			if zero_lag_ranking_stats_filename is not None:
+				self.__write_zero_lag_ranking_stats_file(zero_lag_ranking_stats_filename, verbose = verbose)
 
 			# can't be used anymore
 			del self.coincs_document
