@@ -26,6 +26,8 @@ import cgitb
 import os
 import bisect
 import math
+import json
+import re
 
 # This will run on a web server
 os.environ["MPLCONFIGDIR"] = "/tmp"
@@ -46,6 +48,7 @@ from glue.ligolw import array as ligolw_array
 from glue.ligolw import lsctables
 from glue.ligolw import param as ligolw_param
 from glue.ligolw import utils as ligolw_utils
+from lal import GPSTimeNow
 from pylal import series as lalseries
 from gstlal import far
 cgitb.enable()
@@ -119,7 +122,7 @@ td {
 
 .tabs {
   position: relative;   
-  min-height: 7in; /* This part sucks */
+  min-height: 10in; /* This part sucks */
   clear: both;
   margin: 25px 0;
 }
@@ -233,15 +236,25 @@ class GstlalWebSummary(object):
 
 	def status(self):
 		if self.oldest_data() > 1800:
-			return "<em class=red>SOME DATA OLDER THAN %d seconds</em>" % self.oldest_data() 
+			return 2, "<em class=red>SOME DATA OLDER THAN %d seconds</em>" % self.oldest_data() 
 		if not self.found["latency_history"]:
-			return "<em class=red>NO COINCIDENT EVENTS FOUND!</em>"
+			return 1, "<em class=red>NO COINCIDENT EVENTS FOUND!</em>"
 		if self.missed["latency_history"]:
-			return "<em class=red>%s NODES ARE NOT REPORTING!</em>" % len(self.missed["latency_history"])
+			return 3, "<em class=red>%s NODES ARE NOT REPORTING!</em>" % len(self.missed["latency_history"])
 		lat = [l[-1,1] for l in self.found["latency_history"].values() if l[-1,1] > 180]
 		if lat:
-			return "<em class=red>%s NODES ARE MORE THAN 3 MIN BEHIND!</em>" % len(lat)
-		return "<em class=green>OK</em>"
+			return 1, "<em class=red>%s NODES ARE MORE THAN 3 MIN BEHIND!</em>" % len(lat)
+		lat = [l[-1,1] for l in self.found["latency_history"].values() if l[-1,1] > 300]
+		if lat:
+			return 2, "<em class=red>%s NODES ARE MORE THAN 5 MIN BEHIND!</em>" % len(lat)
+		return 0, "<em class=green>OK</em>"
+
+	def nagios(self):
+		print >>sys.stdout, 'Cache-Control: no-cache, must-revalidate'
+		print >>sys.stdout, 'Expires: Mon, 26 Jul 1997 05:00:00 GMT'
+		print >>sys.stdout, 'Content-type: text/json\r\n'
+		num, txt = self.status()
+		print >>sys.stdout, json.dumps({"nagios_shib_scraper_ver": 0.1, "status_intervals":[{"num_status": num, "txt_status": re.sub('<[^<]+?>', '', txt)}]}, sort_keys=True, indent=4, separators=(',', ': '))
 
 	def latency(self):
 		out = [l[-1,1] for l in self.found["latency_history"].values()]
@@ -430,7 +443,7 @@ class GstlalWebSummary(object):
 		likelihood, ranking_data, nu = found
 		ranking_data.finish()
 		fapfar = far.FAPFAR(ranking_data, livetime = far.get_live_time(nu))
-		fig = plotfar.plot_likelihood_ratio_ccdf(fapfar, (-5, 25), "Noise")
+		fig = plotfar.plot_likelihood_ratio_ccdf(fapfar, (-5, 25))
 		f = StringIO.StringIO()
 		fig.savefig(f, format="png")
 		out = '<img src="data:image/png;base64,' + base64.b64encode(f.getvalue()) + '"></img>'
@@ -488,6 +501,21 @@ class GstlalWebSummary(object):
 				out += self.to_png(fig = fig)
 		return out
 
+	def jointsnrplot(self):
+		out = ""
+		for id in self.registry:
+			likelihood, nu, nu = self.found["likelihood"][id]
+			# FIXME dont hardcode IFOs
+			instruments = (u"H1",u"L1")
+			timenow = GPSTimeNow()
+			horizon_distances = {}
+			for ifo in instruments:
+				horizon_distances[ifo] = likelihood.horizon_history[ifo][timenow]
+			instruments, horizon_distances = likelihood.snr_joint_pdf_keyfunc(instruments, horizon_distances)
+			fig = plotfar.plot_snr_joint_pdf(likelihood, instruments, horizon_distances, 400)
+			out += self.to_png(fig = fig)
+		return out
+
 	def rateplot(self):
 		out = ""
 		for id in self.registry:
@@ -528,7 +556,7 @@ class GstlalWebSummary(object):
 			likelihood, ranking_data, nu = self.found["likelihood"][id]
 			ranking_data.finish()
 			fapfar = far.FAPFAR(ranking_data)
-			fig = plotfar.plot_likelihood_ratio_ccdf(fapfar, (-5, 25), "Noise")
+			fig = plotfar.plot_likelihood_ratio_ccdf(fapfar, (-5, 25))
 			out += self.to_png(fig = fig)
 		return out
 
@@ -537,6 +565,6 @@ class GstlalWebSummary(object):
 		for id in self.registry:
 			likelihood, ranking_data, nu = self.found["likelihood"][id]
 			seglistdicts = self.found["cumulative_segments"][id]
-			fig = plotsegments.plot_segments_history(seglistdicts)
+			fig, h = plotsegments.plot_segments_history(seglistdicts)
 			out += self.to_png(fig = fig)
 		return out
